import pytest
from ewokscore import load_graph
from ewokscore.tests.examples.graphs import get_graph
from ewokscore.tests.examples.graphs import graph_names
from ewokscore.tests.utils.results import assert_execute_graph_default_result


@pytest.mark.parametrize("graph_name", graph_names())
@pytest.mark.parametrize("scheduler", (None, "multithreading", "multiprocessing"))
@pytest.mark.parametrize("scheme", (None, "json"))
def test_examples(engine, graph_name, tmpdir, scheduler, scheme):
    graph, expected = get_graph(graph_name)
    ewoksgraph = load_graph(graph)
    if scheme:
        varinfo = {"root_uri": str(tmpdir), "scheme": scheme}
    else:
        varinfo = None
    if ewoksgraph.is_cyclic or ewoksgraph.has_conditional_links:
        with pytest.raises(RuntimeError):
            engine.execute_graph(graph, scheduler=scheduler, varinfo=varinfo)
    else:
        result = engine.execute_graph(graph, scheduler=scheduler, varinfo=varinfo)
        assert_execute_graph_default_result(ewoksgraph, result, expected, varinfo)



from typing import Any
from typing import List
from typing import Optional
from typing import Union

from dask.distributed import Client
from ewokscore.engine_interface import RawExecInfoType
from ewokscore.engine_interface import WorkflowEngine

from . import bindings


class DaskWorkflowEngine(WorkflowEngine):

    def execute_graph(
        self,
        graph: Any,
        *,
        inputs: Optional[List[dict]] = None,
        load_options: Optional[dict] = None,
        varinfo: Optional[dict] = None,
        execinfo: RawExecInfoType = None,
        task_options: Optional[dict] = None,
        outputs: Optional[List[dict]] = None,
        merge_outputs: Optional[bool] = True,
        # Engine specific:
        scheduler: Union[dict, str, None, Client] = None,
        scheduler_options: Optional[dict] = None,
    ) -> dict:
        return bindings.execute_graph(
            graph,
            inputs=inputs,
            load_options=load_options,
            outputs=outputs,
            merge_outputs=merge_outputs,
            varinfo=varinfo,
            execinfo=execinfo,
            task_options=task_options,
            scheduler=scheduler,
            scheduler_options=scheduler_options,
        )



"""Schedulers with an address and status dashboard."""

from dask.distributed import LocalCluster
from dask_jobqueue import SLURMCluster


def local_scheduler(**kw):
    """
    :param n_workers:
    :param processes: True by default
    :param threads_per_worker:
    :param scheduler_port:
    :param dashboard_address:
    :param worker_dashboard_address:
    """
    kw.setdefault("n_workers", 1)

    cluster = LocalCluster(**kw)

    print("Address:", cluster.scheduler_address)
    print("Dashboard:", cluster.dashboard_link)
    return cluster


def slurm_scheduler(**kw):
    """
    :param address:
    :param n_workers:
    :param minimum_jobs:
    :param maximum_jobs:
    :param cores:
    :param processes:
    :param gpus:
    :param memory:
    :param queue:
    """
    kw.setdefault("walltime", "01:00:00")
    kw.setdefault("cores", 2)
    kw.setdefault("processes", 1)
    kw.setdefault("memory", "8GB")
    job_extra_directives = kw.setdefault("job_extra_directives", list())

    minimum_jobs = kw.pop("minimum_jobs", 0)
    maximum_jobs = kw.pop("maximum_jobs", 0)
    kw.setdefault("n_workers", int(not maximum_jobs))

    gpus = kw.pop("gpus", 0)
    if gpus:
        job_extra_directives.append(f"--gres=gpu:{gpus}")

    cluster = SLURMCluster(**kw)
    if maximum_jobs:
        cluster.adapt(minimum_jobs=minimum_jobs, maximum_jobs=maximum_jobs)

    print("Address:", cluster.scheduler_address)
    print("Dashboard:", cluster.dashboard_link)
    return cluster
